searchNodes=[{"doc":"Trash helps manage soft-deleting Ecto.Schema s by providing convenience functions to update and query for discarded and kept records. Terminology Trash uses a few terms throughout to indicate the state of a record. Here's some quick definitions: Soft-deletion : removing a record by updating an attribute instead of issuing a SQL DELETE Discarded : a record that has been soft-deleted Kept : a record that has not been soft-deleted Restore : reverse a soft-deletion to keep a record","ref":"Trash.html","title":"Trash","type":"module"},{"doc":"Provides query methods for working with records that implement Trash . Schemas should first include Trash.Schema and/or manually add the necessary fields for these to work.","ref":"Trash.Query.html","title":"Trash.Query","type":"module"},{"doc":"Adds trashable fields to select. This ensures that both trashable fields are included in the select statement by using Ecto.Query.select_merge/3 to merge in the fields. For a list of the current trashable fields, see Trash.Schema.trashable_fields/0 . This loads discarded_at from the database and computes the boolean for discarded? from the SQL expression discarded_at IS NOT NULL . Note: Since discarded? is a virtual field, without using this function, it'll be nil by default. Examples iex&gt; Trash.Query . select_trashable ( Post ) |&gt; Repo . all ( ) [ % Post { title : &quot;Hello World&quot; , discarded_at : % DateTime { } , discarded? : true } ] iex&gt; Trash.Query . select_trashable ( Post ) |&gt; Repo . all ( ) [ % Post { title : &quot;Hello World&quot; , discarded_at : nil , discarded? : false } ]","ref":"Trash.Query.html#select_trashable/1","title":"Trash.Query.select_trashable/1","type":"function"},{"doc":"Adds a where clause for returning discarded records. This adds a where clause equivalent to the SQL expression discarded_at IS NOT NULL which denotes a record that has been discarded. Examples iex&gt; Trash.Query . where_discarded ( Post ) |&gt; Repo . all ( ) [ % Post { title : &quot;Hello World&quot; , discarded_at : % DateTime { } , discarded? : nil } ]","ref":"Trash.Query.html#where_discarded/1","title":"Trash.Query.where_discarded/1","type":"function"},{"doc":"Adds a where clause for returning kept records. This adds a where clause equivalent to the SQL expression discarded_at IS NULL which denotes a record that has been kept. Examples iex&gt; Trash.Query . where_kept ( Post ) |&gt; Repo . all ( ) [ % Post { title : &quot;Hello World&quot; , discarded_at : nil , discarded? : nil } ]","ref":"Trash.Query.html#where_kept/1","title":"Trash.Query.where_kept/1","type":"function"},{"doc":"Provides functions for discarding and keeping records and querying for them via Ecto.Repo functions.","ref":"Trash.Repo.html","title":"Trash.Repo","type":"module"},{"doc":"Imports functions from Trash.Repo . It's not required to use this module in order to use Trash . Doing so will import shorthand functions into your app's Repo module with the repo implicitly passed. It's a bit more convenient, but the functions are public on Trash.Repo , so if preferred they can be called directly. # Shorthand with `use` MyRepo . all_discarded ( Post ) # Long form without Trash.Repo . all_discarded ( Post , [ ] , MyRepo ) Options repo - A module reference to an Ecto.Repo ; raises ArgumentError if missing Examples defmodule MyApp.Repo use Ecto.Schema use Trash.Schema , repo : __MODULE__ end","ref":"Trash.Repo.html#__using__/1","title":"Trash.Repo.__using__/1","type":"macro"},{"doc":"Fetches all entries matching the given query that have been discarded. Examples iex&gt; Trash.Repo . all_discarded ( Post , [ ] , MyApp.Repo ) [ % Post { title : &quot;Hello World&quot; , discarded_at : % DateTime { } , discarded? : nil } ]","ref":"Trash.Repo.html#all_discarded/3","title":"Trash.Repo.all_discarded/3","type":"function"},{"doc":"Fetches all entries matching the given query that have been kept. Examples iex&gt; Trash.Repo . all_kept ( Post , [ ] , MyApp.Repo ) [ % Post { title : &quot;Hello World&quot; , discarded_at : nil , discarded? : nil } ]","ref":"Trash.Repo.html#all_kept/3","title":"Trash.Repo.all_kept/3","type":"function"},{"doc":"Updates a record as discarded. This takes either an Ecto.Changeset or an Ecto.Schema struct. If a struct is given a bare changeset is generated first. A change is added to the changeset to set discarded_at to DateTime.utc_now/1 . It calls repo.update/2 to finalize the changes. It returns {:ok, struct} if the struct has been successfully updated or {:error, changeset} if there was an error. Examples iex&gt; Post . changeset ( post , %{ title : &quot;[Archived] Hello, world&quot; } ) |&gt; Trash.Repo . discard ( MyApp.Repo ) { :ok , % Post { title : &quot;[Archived] Hello, world&quot; , discarded_at : % DateTime { } } }","ref":"Trash.Repo.html#discard/2","title":"Trash.Repo.discard/2","type":"function"},{"doc":"Updates a record as discarded. This takes either an Ecto.Changeset or an Ecto.Schema struct. If a struct is given a bare changeset is generated first. A change is added to the changeset to set discarded_at to DateTime.utc_now/1 . It calls repo.update/2 to finalize the changes. Raises Ecto.InvalidChangesetError if the changeset is invalid. Note: since an Ecto.Schema struct can be passed which generates a bare changeset, this will never raise when given a struct. Examples iex&gt; Post . changeset ( post , %{ title : &quot;[Archived] Hello, world&quot; } ) |&gt; Trash.Repo . discard! ( MyApp.Repo ) % Post { title : &quot;[Archived] Hello, world&quot; , discarded_at : % DateTime { } } iex&gt; Post . changeset ( post , %{ } ) |&gt; Trash.Repo . discard! ( MyApp.Repo ) ** (Ecto.InvalidChangesetError)","ref":"Trash.Repo.html#discard!/2","title":"Trash.Repo.discard!/2","type":"function"},{"doc":"Checks if there exists an entry that matches the given query that has been discarded. Returns a boolean. Examples iex&gt; Trash.Repo . discarded? ( post , [ ] , MyApp.Repo ) true","ref":"Trash.Repo.html#discarded?/3","title":"Trash.Repo.discarded?/3","type":"function"},{"doc":"Fetches a single discarded result where the primary key matches the given id . Returns nil if no result was found. Examples iex&gt; Trash.Repo . get_discarded ( Post , 1 , [ ] , MyApp.Repo ) % Post { }","ref":"Trash.Repo.html#get_discarded/4","title":"Trash.Repo.get_discarded/4","type":"function"},{"doc":"Fetches a single discarded result where the primary key matches the given id . Raises Ecto.NoResultsError if no result was found. Examples iex&gt; Trash.Repo . get_discarded! ( Post , 1 , [ ] , MyApp.Repo ) % Post { } iex&gt; Trash.Repo . get_discarded! ( Post , 2 , [ ] , MyApp.Repo ) ** (Ecto.NoResultsError)","ref":"Trash.Repo.html#get_discarded!/4","title":"Trash.Repo.get_discarded!/4","type":"function"},{"doc":"Fetches a single discarded result from the query. Returns nil if no result was found or raises Ecto.MultipleResultsError if more than one entry. Examples iex&gt; Trash.Repo . get_discarded_by ( Post , [ title : &quot;Hello World&quot; ] , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : % DateTime { } }","ref":"Trash.Repo.html#get_discarded_by/4","title":"Trash.Repo.get_discarded_by/4","type":"function"},{"doc":"Fetches a single discarded result from the query. Raises Ecto.MultipleResultsError if more than one result. Raises Ecto.NoResultsError if no result was found. Examples iex&gt; Trash.Repo . get_discarded_by! ( Post , [ title : &quot;Hello World&quot; ] , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : % DateTime { } } iex&gt; Trash.Repo . get_discarded_by! ( Post , [ title : &quot;Hello World&quot; ] , [ ] , MyApp.Repo ) ** (Ecto.NoResultsError)","ref":"Trash.Repo.html#get_discarded_by!/4","title":"Trash.Repo.get_discarded_by!/4","type":"function"},{"doc":"Fetches a single kept result where the primary key matches the given id . Returns nil if no result was found. Examples iex&gt; Trash.Repo . get_kept ( Post , 1 , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : nil }","ref":"Trash.Repo.html#get_kept/4","title":"Trash.Repo.get_kept/4","type":"function"},{"doc":"Fetches a single kept result where the primary key matches the given id . Raises Ecto.NoResultsError if no result was found. Examples iex&gt; Trash.Repo . get_kept! ( Post , 1 , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : nil } iex&gt; Trash.Repo . get_kept! ( Post , 2 , [ ] , MyApp.Repo ) ** (Ecto.NoResultsError)","ref":"Trash.Repo.html#get_kept!/4","title":"Trash.Repo.get_kept!/4","type":"function"},{"doc":"Fetches a single kept result from the query. Returns nil if no result was found or raises Ecto.MultipleResultsError if more than one entry. Examples iex&gt; Trash.Repo . get_kept_by ( Post , title : &quot;Hello World&quot; , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : nil }","ref":"Trash.Repo.html#get_kept_by/4","title":"Trash.Repo.get_kept_by/4","type":"function"},{"doc":"Fetches a single kept result from the query. Raises Ecto.MultipleResultsError if more than one result. Raises Ecto.NoResultsError if no result was found. Examples iex&gt; Trash.Repo . get_kept_by! ( Post , title : &quot;Hello World&quot; , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : % DateTime { } } iex&gt; Trash.Repo . get_kept_by! ( Post , title : &quot;Not Written&quot; , [ ] , MyApp.Repo ) ** (Ecto.NoResultsError)","ref":"Trash.Repo.html#get_kept_by!/4","title":"Trash.Repo.get_kept_by!/4","type":"function"},{"doc":"Checks if there exists an entry that matches the given query that has been kept. Returns a boolean. Examples iex&gt; Trash.Repo . kept? ( post , [ ] , MyApp.Repo ) true","ref":"Trash.Repo.html#kept?/3","title":"Trash.Repo.kept?/3","type":"function"},{"doc":"Fetches a single discarded result from the query. Returns nil if no result was found or raises Ecto.MultipleResultsError if more than one entry. Examples iex&gt; Trash.Repo . one_discarded ( Post , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : % DateTime { } }","ref":"Trash.Repo.html#one_discarded/3","title":"Trash.Repo.one_discarded/3","type":"function"},{"doc":"Fetches a single discarded result from the query. Raises Ecto.MultipleResultsError if more than one entry. Raises Ecto.NoResultsError if no result was found. Examples iex&gt; Trash.Repo . one_discarded! ( Post , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : % DateTime { } } iex&gt; Trash.Repo . one_discarded! ( Post , [ ] , MyApp.Repo ) ** (Ecto.NoResultsError)","ref":"Trash.Repo.html#one_discarded!/3","title":"Trash.Repo.one_discarded!/3","type":"function"},{"doc":"Fetches a single kept result from the query. Returns nil if no result was found or raises Ecto.MultipleResultsError if more than one entry. Examples iex&gt; Trash.Repo . one_kept ( Post , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : nil }","ref":"Trash.Repo.html#one_kept/3","title":"Trash.Repo.one_kept/3","type":"function"},{"doc":"Fetches a single kept result from the query. Raises Ecto.MultipleResultsError if more than one entry. Raises Ecto.NoResultsError if no result was found. Examples iex&gt; Trash.Repo . one_kept! ( Post , [ ] , MyApp.Repo ) % Post { title : &quot;Hello World&quot; , discarded_at : nil } iex&gt; Trash.Repo . one_kept! ( Post , [ ] , MyApp.Repo ) ** (Ecto.NoResultsError)","ref":"Trash.Repo.html#one_kept!/3","title":"Trash.Repo.one_kept!/3","type":"function"},{"doc":"Updates a record as kept. This takes either an Ecto.Changeset or an Ecto.Schema struct. If a struct is given a bare changeset is generated first. A change is added to the changeset to set discarded_at to nil . It calls repo.update/2 to finalize the changes. It returns {:ok, struct} if the struct has been successfully updated or {:error, changeset} if there was an error. Examples iex&gt; Post . changeset ( post , %{ title : &quot;Hello, world&quot; } ) |&gt; Trash.Repo . restore ( MyApp.Repo ) { :ok , % Post { title : &quot;Hello, world&quot; , discarded_at : nil } }","ref":"Trash.Repo.html#restore/2","title":"Trash.Repo.restore/2","type":"function"},{"doc":"Updates a record as kept. This takes either an Ecto.Changeset or an Ecto.Schema struct. If a struct is given a bare changeset is generated first. A change is added to the changeset to set discarded_at to nil . It calls repo.update/2 to finalize the changes. Raises Ecto.InvalidChangesetError if the changeset is invalid. Note: since an Ecto.Schema struct can be passed which generates a bare changeset, this will never raise when given a struct. Examples iex&gt; Post . changeset ( post , %{ title : &quot;[Archived] Hello, world&quot; } ) |&gt; Trash.Repo . restore! ( MyApp.Repo ) % Post { title : &quot;[Archived] Hello, world&quot; , discarded_at : nil } iex&gt; Post . changeset ( post , %{ } ) |&gt; Trash.Repo . restore! ( MyApp.Repo ) ** (Ecto.InvalidChangesetError)","ref":"Trash.Repo.html#restore!/2","title":"Trash.Repo.restore!/2","type":"function"},{"doc":"Provides functions for integrating Trash with Ecto.Schema . Example defmodule MyApp.Post do use Ecto.Schema use Trash.Schema schema &quot;posts&quot; do field ( :title , :string ) trashable_fields ( ) end end","ref":"Trash.Schema.html","title":"Trash.Schema","type":"module"},{"doc":"Imports functions from Trash.Schema . Currently no options are available.","ref":"Trash.Schema.html#__using__/1","title":"Trash.Schema.__using__/1","type":"macro"},{"doc":"Declares fields on Ecto.Schema necessary for Trash . This is a macro that can be used inside of an Ecto.Schema.schema/2 block to add the necessary fields. Fields discarded_at - :utc_datetime discarded? - :boolean (virtual) Note: under normal circumstances, discarded? will be nil since it's not possible to load a virtual field in Ecto. Instead, use Trash.Query.select_trashable/1 to hydrate this field with a computed value from the database.","ref":"Trash.Schema.html#trashable_fields/0","title":"Trash.Schema.trashable_fields/0","type":"macro"},{"doc":"♻️ Trash Simple soft deletes for Ecto","ref":"readme.html","title":"♻️  Trash","type":"extras"},{"doc":"Trash is available on Hex . This package can be installed by adding trash to your list of dependencies in mix.exs : def deps do [ { :trash , &quot;~&gt; 0.1.0&quot; } ] end","ref":"readme.html#installation","title":"♻️  Trash - Installation","type":"extras"},{"doc":"Check the documentation for complete details. Trash helps manage soft-deleting Ecto.Schema s by providing convenience functions to update and query for discarded and kept records. Terminology Trash uses a few terms throughout to indicate the state of a record. Here are some quick definitions: Soft-deletion : removing a record by updating an attribute instead of issuing a SQL DELETE Discarded : a record that has been soft-deleted Kept : a record that has not been soft-deleted Restore : reverse a soft-deletion to keep a record Getting Started Trash is opt-in on individual Ecto.Schema s. To start marking schemas as trashable, first add the required trashable fields: mix ecto.gen.migration add_trashable_to_posts defmodule MyApp.Repo.Migrations.AddTrashableToPosts do use Ecto.Migration def change do alter ( table ( :posts ) ) do add ( :discarded_at , :utc_datetime ) end create ( index ( :posts , :discarded_at ) ) end end Then declare the fields on your schema. You can do this manually or use the convenience functions in Trash.Schema : defmodule MyApp.Posts.Post do use Ecto.Schema use Trash.Schema schema &quot;posts&quot; do field ( :title , :string ) trashable_fields ( ) end end Next, import Trash by using it in your MyApp.Repo . defmodule MyApp.Repo do use Ecto.Repo , otp_app : :my_app , adapter : Ecto.Adapters.Postgres use Trash.Repo , repo : __MODULE__ end This generates shorthand functions with the repo implicitly passed. However, it's not required to call use . If preferred you can call the functions directly on Trash.Repo by passing the Ecto.Repo manually. It's a bit more convenient with use , though. # Shorthand with `use` MyRepo . all_discarded ( Post ) # Long form without MyRepo . all_discarded ( Post , [ ] , MyRepo ) Soft-deleting and Restoring The functions discard and restore will soft-delete and restore records, respectively. alias MyApp.Posts alias MyApp.Repo post = Posts . get_last_post! { :ok , post } = Repo . discard ( post ) # =&gt; %Post{discarded_at: %DateTime{}} post = Repo . restore ( post ) # =&gt; %Post{discarded_at: nil} These call out to the repo's update function. This means a SQL UPDATE has been issued and the returned schema has updated trashable fields. These functions also have bang versions, which unwrap the return tuple and raise on error. Note: when passing a struct instead of a changeset, the bang versions of these will never raise an error. Querying Trash provides discarded and kept variations of the following Ecto.Repo functions: all exists? get get! get_by get_by! one one! The variations are postfixed with discarded and kept (with the exception of exists? which is replaced by discarded? and kept? ) and modify the passed-in queryable to add a WHERE condition to only return discarded or kept records. Trash also provides helper where functions that can be used in conjunction with Ecto.Query . import Ecto.Query alias MyApp.Posts.Post from ( p in Post ) |&gt; Trash.Query . where_discarded ( ) |&gt; Repo . all ( ) There is also a function that merges in the trashable fields into the select statement to always ensure they are returned. It also hydrates discarded? with a computed SQL value. import Ecto.Query alias MyApp.Posts.Post alias MyApp.Repo Post |&gt; Trash.Query . where_discarded ( ) |&gt; Repo . all ( ) |&gt; Trash.Query . select_trashable ( )","ref":"readme.html#usage","title":"♻️  Trash - Usage","type":"extras"},{"doc":"Contributions are welcome! To make changes, clone the repo, make sure tests pass, and then open a PR on GitHub. git clone https://github.com/newaperio/trash.git cd trash mix deps.get mix test","ref":"readme.html#contributing","title":"♻️  Trash - Contributing","type":"extras"},{"doc":"Trash is Copyright © 2020 NewAperio. It is free software, and may be redistributed under the terms specified in the LICENSE file.","ref":"readme.html#license","title":"♻️  Trash - License","type":"extras"},{"doc":"Trash is built by NewAperio, LLC. NewAperio is a web and mobile design and development studio. We offer expert Elixir and Phoenix development as part of our portfolio of services. Get in touch to see how our team can help you.","ref":"readme.html#about-newaperio","title":"♻️  Trash - About NewAperio","type":"extras"},{"doc":"MIT License Copyright (c) 2020 NewAperio, LLC Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]